/*

Anleitung für zukünftige Änderungen an den Sets:

+ FUNKTION/ZEILE:
  ÄNDERUNG

+ Define constants for pins:
  Evtl. Taster für weitere Sets definieren
+ Definieren der Notenwerte für die einzelnen Sets:
  Mit const int setWeiteresSet1....4 - die einzelnen Noten definieren 
+ Chord intervals matrix:
  Diese Matrix entsprechend anpassen/ergänzen
+ PROGRAMM SETUP:
  setSelectPinXX - Die Pinzuordnung eines weiteren Tasters definieren 
+ Abfrage der Bedienknöpfe:
  setSelectXXState - Hier werden die Auswahltaster abgefragt und das aktuelle Set definiert (am besten diesen Codebereich mit der KI ändern lassen)
+ Notenausgabe:
  Die einzelnen Cases anpassen/ergänzen
+ Stop der Notenausgabe:
  Die einzelnen Cases anpassen/ergänzen
  
*/


// Include required libraries
#include <MIDI.h>     // For 5-pin MIDI Out
#include <MIDIUSB.h>  // For USB MIDI

// Define constants for pins
const int pedalPins[] = { 2, 15, 13, 14, 3, 16 };  // 6 foot pedals
const int setSelectPin1 = 7;                       // Taster für Dkurd sets
const int setSelectPin2 = 8;                       // Taster für Fislowp sets
const int setSelectPin3 = 10;                      // Taster für Chromatic sets
const int setSelectPin4 = 9;                       // Taster für Weitere Pan sets
const int modePotPin = A6;                         // Pot for change the modi
const int transposePotPin = A0;                    // Pot for transpose over 4 octaves
const int potPins[] = { A4, A1, A2, A3, A5 };      // 5 Potentiometers for CC MIDI

// MIDI channel
const int midiChannel = 1;

// Definieren der Notenwerte für die einzelnen Sets

// D kurd
// MIDI note sets Bb=46 C=48 D=50 F=53 G=55 A=57 Bb=58 C=60
const int setDkurd1[] = { 46, 48, 50, 53, 55, 57 };  // Bb, C, Dm, F, Gm, Am - 4, 5, 6, 1 - Tief nach hoch
const int setDkurd2[] = { 48, 50, 53, 58, 57, 60 };  //  C, Dm, F, Bb, Am, C - 5, 6, 1, 4 - Tief nach hoch
const int setDkurd3[] = { 50, 53, 58, 60, 46, 48 };  //  Dm, F, Bb, C, Bb, C - 6, 1, 4, 5 - Tief nach hoch
const int setDkurd4[] = { 53, 48, 50, 46, 58, 60 };  //  F, C, Dm, Bb, Bb, C - 1, 5, 6, 4 - Liedfolge

// Fislowpygmy
// MIDI note sets D=50 E=52 F#=54 A=57 B=59 C#=61 D=62
const int setFislowp1[] = { 50, 52, 54, 57, 59, 61 };  // D, E, F#m, A, Bm, C#m - 4, 5, 6, 1 - Tief nach hoch
const int setFislowp2[] = { 52, 54, 57, 50, 61, 50 };  // E, F#m, A, D, Bm, C#m - 5, 6, 1, 4 - Tief nach hoch
const int setFislowp3[] = { 54, 57, 50, 52, 50, 52 };  // F#m, A, D, E, Bm, C#m - 6, 1, 4, 5 - Tief nach hoch
const int setFislowp4[] = { 50, 52, 54, 59, 49, 59 };  // D, E, F#m, Bm

// Weitere Handppan
const int setWeiterePan1[] = { 60, 62, 64, 67, 70, 71 };  // C, D, E, G Test
const int setWeiterePan2[] = { 62, 64, 67, 60, 70, 71 };  // D, E, G, C Test
const int setWeiterePan3[] = { 64, 67, 60, 62, 70, 71 };  // E, G, C, D Test
const int setWeiterePan4[] = { 67, 60, 62, 64, 70, 71 };  // G, C, D, E Test

// Chromatic
const int setChromaticUp[] = { 48, 49, 50, 51, 52, 53 };    // Chromatic up
const int setChromaticDown[] = { 51, 50, 49, 48, 49, 48 };  // Chromatic down

// Chord intervals matrix
const int intervals[14][6] = {
  { 4, 4, 3, 4, 4, 4 },  // Dkurd1
  { 4, 3, 4, 4, 3, 3 },  // Dkurd2
  { 3, 4, 4, 4, 3, 4 },  // Dkurd3
  { 4, 4, 3, 4, 3, 4 },  // Dkurd4
  { 4, 4, 3, 4, 4, 4 },  // Fislowp1
  { 4, 3, 4, 4, 3, 4 },  // Fislowp2
  { 3, 4, 4, 4, 3, 4 },  // Fislowp3
  { 4, 4, 3, 4, 3, 4 },  // Fislowp4
  { 4, 4, 3, 4, 4, 4 },  // WeiterePan1
  { 4, 3, 4, 4, 3, 4 },  // WeiterePan2
  { 3, 4, 4, 4, 3, 4 },  // WeiterePan3
  { 4, 4, 3, 4, 3, 4 },  // WeiterePan4
  { 4, 4, 4, 4, 4, 4 },  // ChromaticUp
  { 4, 4, 4, 4, 4, 4 },  // ChromaticDown
};

// Current state variables
int currentSet = 0;
int currentMode = 0;
int transposeState = 0;
int ModeState = 0;
bool modeButtonLastState = HIGH;
unsigned long lastButtonPress = 0;

int lastPotValues[4] = { -1, -1, -1, -1 };  // Letzte Werte der CC-Potis
unsigned long lastPotSendTime = 0;

// MIDI interface for 5-pin MIDI out via Serial1 (Pin 1 als TX)
MIDI_CREATE_INSTANCE(HardwareSerial, Serial1, MIDI);

// PROGRAMM SETUP
void setup() {
  for (int i = 0; i < 6; i++) {
    pinMode(pedalPins[i], INPUT_PULLUP);
  }
  pinMode(setSelectPin1, INPUT_PULLUP);
  pinMode(setSelectPin2, INPUT_PULLUP);
  pinMode(setSelectPin3, INPUT_PULLUP);
  pinMode(setSelectPin4, INPUT_PULLUP);
  pinMode(modePotPin, INPUT_PULLUP);
  pinMode(transposePotPin, INPUT);

  Serial1.begin(31250);  // MIDI Baudrate für 5-Pin MIDI
  MIDI.begin(MIDI_CHANNEL_OMNI);
}

// BEGINN DER PROGRAMMSCHLEIFE
void loop() {
  handleButtons();                 // buttons
  handlePedals();                  // foot pedals
  handlePotentiometers();          // CC pots
  handleTransposePotentiometer();  // transpose with pot
  handleModePotentiometer();       // mode select with pot
}

// Abfrage der Bedienknöpfe
void handleButtons() {
  unsigned long currentTime = millis();
  if (currentTime - lastButtonPress < 100) return;

  bool setSelect1State = digitalRead(setSelectPin1);
  if (setSelect1State == LOW) {
    currentSet = (currentSet < 4) ? (currentSet + 1) % 4 : 0;  // Dkurd sets 1-4
    lastButtonPress = currentTime;
  }

  bool setSelect2State = digitalRead(setSelectPin2);
  if (setSelect2State == LOW) {
    if (currentSet < 4 || currentSet > 7) {
      currentSet = 4;  // Starte bei Fislowp 1
    } else {
      currentSet = (currentSet - 4 + 1) % 4 + 4;  // Fislowp sets 1-4
    }
    lastButtonPress = currentTime;
  }

  bool setSelect3State = digitalRead(setSelectPin3);
  if (setSelect3State == LOW) {
    if (currentSet < 8 || currentSet > 11) {
      currentSet = 8;  // Starte bei WeiterePan 1
    } else {
      currentSet = (currentSet - 4 + 1) % 4 + 8;  // WeiterePan sets 1-4
    }
    lastButtonPress = currentTime;
  }

  bool setSelect4State = digitalRead(setSelectPin4);
  if (setSelect4State == LOW) {
    if (currentSet < 12 || currentSet > 13) {
      currentSet = 12;  // Starte bei ChromaticUp
    } else {
      currentSet = (currentSet + 1) % 2 + 12;  // Chromatic Up/Down
    }
    lastButtonPress = currentTime;
  }
}

// Abfrage des Transpose Potentiometers
void handleTransposePotentiometer() {  // transpose steps -12, 0, +12, +24
  int potValue = analogRead(transposePotPin);
  int newTransposeState;
  if (potValue < 256) {
    newTransposeState = -12;
  } else if (potValue < 512) {
    newTransposeState = 0;
  } else if (potValue < 768) {
    newTransposeState = 12;
  } else {
    newTransposeState = 24;
  }

  if (newTransposeState != transposeState) {
    transposeState = newTransposeState;
  }
}

// Abfrage des Mode Potentiometers
void handleModePotentiometer() {  // Mode change
  int potValue = analogRead(modePotPin);
  int newModeState;
  if (potValue < 130) {  // Mode 1 = Root
    newModeState = 0;
  } else if (potValue < 300) {  // Mode 2 = Root + 8th
    newModeState = 1;
  } else if (potValue < 610) {  // Mode 3 = Root + 10th
    newModeState = 2;
  } else if (potValue < 760) {  // Mode 4 = Root + 5th + 10th
    newModeState = 3;
  } else if (potValue < 970) {  // Mode 5 = Root + 10th + 12th
    newModeState = 4;
  } else {  // Mode 6 = Root + 10th + 12th + 16th
    newModeState = 5;
  }

  if (newModeState != currentMode) {
    currentMode = newModeState;
  }
}

// Überwachung der Fußpedale
void handlePedals() {
  static bool lastPedalState[6] = { HIGH, HIGH, HIGH, HIGH, HIGH, HIGH };
  for (int i = 0; i < 6; i++) {
    int pedalState = digitalRead(pedalPins[i]);
    if (pedalState == LOW && lastPedalState[i] == HIGH) {
      playNotes(i);
    } else if (pedalState == HIGH && lastPedalState[i] == LOW) {
      stopNotes(i);
    }
    lastPedalState[i] = pedalState;
  }
}

// Festlegung des Intervals entsprechend der Chord intervals matrix
int chordInterval(int pedalIndex) {
  return intervals[currentSet][pedalIndex];
}

// Notenausgabe
void playNotes(int pedalIndex) {
  int baseNote;
  switch (currentSet) {
    case 0: baseNote = setDkurd1[pedalIndex]; break;
    case 1: baseNote = setDkurd2[pedalIndex]; break;
    case 2: baseNote = setDkurd3[pedalIndex]; break;
    case 3: baseNote = setDkurd4[pedalIndex]; break;
    case 4: baseNote = setFislowp1[pedalIndex]; break;
    case 5: baseNote = setFislowp2[pedalIndex]; break;
    case 6: baseNote = setFislowp3[pedalIndex]; break;
    case 7: baseNote = setFislowp4[pedalIndex]; break;
    case 8: baseNote = setWeiterePan1[pedalIndex]; break;
    case 9: baseNote = setWeiterePan2[pedalIndex]; break;
    case 10: baseNote = setWeiterePan3[pedalIndex]; break;
    case 11: baseNote = setWeiterePan4[pedalIndex]; break;
    case 12: baseNote = setChromaticUp[pedalIndex]; break;
    case 13: baseNote = setChromaticDown[pedalIndex]; break;
  }

  baseNote += transposeState;

  if (currentMode == 0) {
    sendNoteOn(baseNote);  // 1 Root
  } else if (currentMode == 1) {
    sendNoteOn(baseNote);       // 2 Root
    sendNoteOn(baseNote + 12);  // 2 8th
  } else if (currentMode == 2) {
    sendNoteOn(baseNote);                                   // 3 Root
    sendNoteOn(baseNote + 12 + chordInterval(pedalIndex));  // 3 10th
  } else if (currentMode == 3) {
    sendNoteOn(baseNote);                                   // 4 Root
    sendNoteOn(baseNote + 7);                               // 4 5th
    sendNoteOn(baseNote + 12 + chordInterval(pedalIndex));  // 4 10th
  } else if (currentMode == 4) {
    sendNoteOn(baseNote);                                   // 5 Root
    sendNoteOn(baseNote + 12 + chordInterval(pedalIndex));  // 5 10th
    sendNoteOn(baseNote + 7 + 12);                          // 5 12th
  } else if (currentMode == 5) {
    sendNoteOn(baseNote);                                   // 6 Root
    sendNoteOn(baseNote + 12 + chordInterval(pedalIndex));  // 6 10th
    sendNoteOn(baseNote + 7 + 12);                          // 6 12th
    sendNoteOn(baseNote + 12 + 12);                         // 6 16th
  }
}

// Stop der Notenausgabe
void stopNotes(int pedalIndex) {
  int baseNote;
  switch (currentSet) {
    case 0: baseNote = setDkurd1[pedalIndex]; break;
    case 1: baseNote = setDkurd2[pedalIndex]; break;
    case 2: baseNote = setDkurd3[pedalIndex]; break;
    case 3: baseNote = setDkurd4[pedalIndex]; break;
    case 4: baseNote = setFislowp1[pedalIndex]; break;
    case 5: baseNote = setFislowp2[pedalIndex]; break;
    case 6: baseNote = setFislowp3[pedalIndex]; break;
    case 7: baseNote = setFislowp4[pedalIndex]; break;
    case 8: baseNote = setWeiterePan1[pedalIndex]; break;
    case 9: baseNote = setWeiterePan2[pedalIndex]; break;
    case 10: baseNote = setWeiterePan3[pedalIndex]; break;
    case 11: baseNote = setWeiterePan4[pedalIndex]; break;
    case 12: baseNote = setChromaticUp[pedalIndex]; break;
    case 13: baseNote = setChromaticDown[pedalIndex]; break;
  }

  baseNote += transposeState;

  if (currentMode == 0) {
    sendNoteOff(baseNote);  // 1 Root
  } else if (currentMode == 1) {
    sendNoteOff(baseNote);       // 2 Root
    sendNoteOff(baseNote + 12);  // 2 8th
  } else if (currentMode == 2) {
    sendNoteOff(baseNote);                                   // 3 Root
    sendNoteOff(baseNote + 12 + chordInterval(pedalIndex));  // 3 10th
  } else if (currentMode == 3) {
    sendNoteOff(baseNote);                                   // 4 Root
    sendNoteOff(baseNote + 7);                               // 4 5th
    sendNoteOff(baseNote + 12 + chordInterval(pedalIndex));  // 4 10th
  } else if (currentMode == 4) {
    sendNoteOff(baseNote);                                   // 5 Root
    sendNoteOff(baseNote + 12 + chordInterval(pedalIndex));  // 5 10th
    sendNoteOff(baseNote + 7 + 12);                          // 5 12th
  } else if (currentMode == 5) {
    sendNoteOff(baseNote);                                   // 6 Root
    sendNoteOff(baseNote + 12 + chordInterval(pedalIndex));  // 6 10th
    sendNoteOff(baseNote + 7 + 12);                          // 6 12th
    sendNoteOff(baseNote + 12 + 12);                         // 6 16th
  }
}

// read pots for MIDI-CC
void handlePotentiometers() {
  unsigned long currentTime = millis();
  if (currentTime - lastPotSendTime < 50) return;

  for (int i = 0; i < 5; i++) {
    int potValue = analogRead(potPins[i]);
    potValue = map(potValue, 0, 1023, 0, 127);
    if (potValue != lastPotValues[i]) {
      sendControlChange(i + 1, potValue);
      lastPotValues[i] = potValue;
    }
  }
  lastPotSendTime = currentTime;
}

// Funktion zum Senden von NoteOn über MIDI
void sendNoteOn(int note) {
  MIDI.sendNoteOn(note, 127, midiChannel);
  midiEventPacket_t noteOn = { 0x09, 0x90 | (midiChannel - 1), note, 127 };
  MidiUSB.sendMIDI(noteOn);
  MidiUSB.flush();
}

// Funktion zum Senden von NoteOff über MIDI
void sendNoteOff(int note) {
  MIDI.sendNoteOff(note, 0, midiChannel);
  midiEventPacket_t noteOff = { 0x08, 0x80 | (midiChannel - 1), note, 0 };
  MidiUSB.sendMIDI(noteOff);
  MidiUSB.flush();
}

// Funktion zum Senden von MIDI-CC-Daten
void sendControlChange(byte control, byte value) {
  MIDI.sendControlChange(control, value, midiChannel);
  midiEventPacket_t cc = { 0x0B, 0xB0 | (midiChannel - 1), control, value };
  MidiUSB.sendMIDI(cc);
  MidiUSB.flush();
}
